@@ -334,7 +334,10 @@ class TestUnixSocketController:
             )
             sock.connect(str(self.sockfile))
             if ssl_context:
-                sock = stk.enter_context(ssl_context.wrap_socket(sock))
+                context = ssl.create_default_context(ssl.Purpose.SERVER_AUTH)
+                context.check_hostname = False
+                context.verify_mode = ssl.CERT_NONE
+                sock = stk.enter_context(context.wrap_socket(sock))
                 time.sleep(0.1)
             resp = sock.recv(1024)
             assert resp.startswith(b"220 ")
